/**
 * 'fmt' and 'parse' utility object, input formatter classes.
 * @module data/input-formatter
 */
import { fmt, numberDefaults } from "./format";
import { calendarDefaults } from "../components/use-calendar";
/*
                              dP
                              88
 88d888b. dP    dP 88d8b.d8b. 88d888b. .d8888b. 88d888b.
 88'  `88 88    88 88'`88'`88 88'  `88 88ooood8 88'  `88
 88    88 88.  .88 88  88  88 88.  .88 88.  ... 88
 dP    dP `88888P' dP  dP  dP 88Y8888' `88888P' dP

*/
export class NumberFmt {
    constructor(prec, sep) {
        if (!prec) {
            this.precision = 0;
        }
        else {
            if (Math.abs(prec) > 6) {
                throw new Error(`Unsupported precision number ${prec}`);
            }
            this.precision = prec;
        }
        this.separator = sep !== undefined ? sep : numberDefaults.read().thousandSeparator ?? " ";
    }
    // TODO: move format / parse number into format.ts
    parse(val) {
        val = val.replace(/\s/g, ""); // remove whitespace
        val = val.split(this.separator).join(""); // replace all
        const num = parseFloat(val);
        if (Number.isNaN(num)) {
            return undefined;
        }
        const pwr = 10.0 ** -this.precision;
        console.log("PWR", pwr, val, num, Math.round(num / pwr) * pwr);
        return Math.round(num / pwr) * pwr;
    }
    format(val) {
        if (val === undefined) {
            return "";
        }
        if (typeof val !== "number") {
            throw new Error("numberFormatter expects number as data type, got " + typeof val);
        }
        let str = val.toFixed(Math.max(this.precision, 0)); // `${val}`;
        const len = str.length;
        const firstPos = this.precision > 0 ? this.precision + 1 : 0;
        for (let n = 3 + firstPos; n < len; n += 3) {
            str = str.substring(0, len - n) + this.separator + str.substring(len - n);
        }
        return str;
    }
}
/*

 88d8b.d8b. .d8888b. 88d888b. .d8888b. dP    dP
 88'`88'`88 88'  `88 88'  `88 88ooood8 88    88
 88  88  88 88.  .88 88    88 88.  ... 88.  .88
 dP  dP  dP `88888P' dP    dP `88888P' `8888P88
                                            .88
                                        d8888P
*/
/** Encode . * + ? ^ $ { } ( ) | [ ] \ to do litral match in regex */
function escapeRegExp(val) {
    return val.replace(/[.*+?^${}()|[\]\\]/g, "\\$&").trim();
}
export class MoneyFmt {
    constructor(prec, pref, suff) {
        const def = numberDefaults.read();
        const moneyPrec = prec !== undefined ? prec : def.moneyPrecision ?? 2;
        this.numFormatter = new NumberFmt(moneyPrec);
        this.prefix = pref !== undefined ? pref : def.moneyPrefix ?? "$";
        this.suffix = suff !== undefined ? suff : def.moneySuffix ?? "";
        this.prefixRegex = new RegExp("^\\s*" + escapeRegExp(this.prefix) + "\\s*");
        this.suffixRegex = new RegExp("\\s*" + escapeRegExp(this.suffix) + "\\s*$");
    }
    // TODO: add precision
    parse(val) {
        const numStr = val.replace(this.prefixRegex, "").replace(this.suffixRegex, "");
        return this.numFormatter.parse(numStr);
    }
    format(val) {
        const str = this.numFormatter.format(val);
        console.log("fmt", str);
        if (!str) {
            return "";
        }
        return this.prefix + str + this.suffix;
    }
}
export class TextFmt {
    //TODO: ascii only, id, id_, id_-, no tags, no-sq, no-dq, max-len
    constructor(caseMode) {
        this.caseMode = caseMode;
    }
    format(val) {
        if (typeof val !== "string") {
            throw new Error("CaseFormatter expects string as data type");
        }
        switch (this.caseMode) {
            case "upper":
                return val.trim().toUpperCase();
            case "lower":
                return val.trim().toLowerCase();
            case "title":
                return val
                    .trim()
                    .split(" ")
                    .map((s) => s.charAt(0).toUpperCase() + s.substring(1).toLowerCase())
                    .join(" ");
        }
    }
}
/*
 dP oo            dP
 88               88
 88 dP .d8888b. d8888P
 88 88 Y8ooooo.   88
 88 88       88   88
 dP dP `88888P'   dP

*/
export class ListFmt {
    constructor(sep) {
        if (sep === "") {
            throw new Error("List separator can't be empty");
        }
        this.separator = sep !== undefined ? sep : numberDefaults.read().listSeparator;
    }
    parse(val) {
        if (!val) {
            return [];
        }
        let sep = this.separator.trim();
        if (sep === "") {
            sep = " ";
        }
        return val.split(this.separator).map((item) => item.trim());
    }
    format(val) {
        if (!Array.isArray(val)) {
            throw new Error("listFormatter expects string[] as data type, got " + typeof val);
        }
        const str = val.reduce((res, curr) => res + this.separator + curr, "");
        if (str === "") {
            return "";
        }
        return str.substring(this.separator.length);
    }
}
/*
       dP            dP
       88            88
 .d888b88 .d8888b. d8888P .d8888b.
 88'  `88 88'  `88   88   88ooood8
 88.  .88 88.  .88   88   88.  ...
 `88888P8 `88888P8   dP   `88888P'

 */
export function parseDate(val) {
    if (!val) {
        return undefined;
    }
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-explicit-any, @typescript-eslint/typedef
    const jQuery = window.jQuery;
    if (!jQuery) {
        throw new Error("jQuery in not initialized");
    }
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/typedef
    const dateParser = jQuery.fn.calendar.settings.parser.date;
    if (!dateParser) {
        throw new Error("Semantic UI calendar in not initialized");
    }
    return dateParser(val, { type: "date" });
}
export function formatDate(val) {
    if (val === undefined) {
        return "";
    }
    if (!(val instanceof Date)) {
        throw new Error("dateFormatter expects Date as data type, got " + typeof val);
    }
    if (calendarDefaults.read().formatter?.date) {
        return fmt.isoDate(val);
        // return dateFormatDefaults.formatter?.date(val);
    }
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-explicit-any, @typescript-eslint/typedef
    const jQuery = window.jQuery;
    if (!jQuery) {
        throw new Error("jQuery in not initialized");
    }
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/typedef
    const dateFormatter = jQuery.fn.calendar.settings.formatter.date;
    if (!dateFormatter) {
        throw new Error("Semantic UI calendar in not initialized");
    }
    return dateFormatter(val, { type: "date" });
}
export class DateFmt {
    //TODO: add constructor withs ettings
    //TODO: default formatter (copy)
    //TODO: defaukt formatter (live)
    parse(val) {
        return parseDate(val);
    }
    format(val) {
        return fmt.isoDate(val);
        // return dateFormatDefaults.formatter?.date!(val as Date) ?? "";
        // return formatDate(val);
    }
}
